
class Aggregator(object):
  AGGREGATOR_COMPLETED_EXCHANGES = "AggregatorCompletedExchanges"

  def __init__(self, eip_context, correlation_expression, aggregation_strategy, completion_expression_list):
    self.__aggregation_strategy = aggregation_strategy
    self.__correlation_expression = correlation_expression
    self.__completion_expression_list = completion_expression_list
    self.__aggregation_state = eip_context.manager.dict()
    self.__lock = eip_context.manager.RLock()

  def aggregate(self, new_exchange):
    correlation_id = self.__correlation_expression.evaluate(new_exchange)
    if correlation_id is not None:
      self.__lock.acquire()
      try:
        old_exchange = self.__aggregation_state.get(correlation_id)
        count = 0 if old_exchange is None else old_exchange.get_header(Aggregator.AGGREGATOR_COMPLETED_EXCHANGES)
        exchange = self.__aggregation_strategy.aggregate(old_exchange, new_exchange)
        exchange.set_header(Aggregator.AGGREGATOR_COMPLETED_EXCHANGES, count+1)
        self.__aggregation_state[correlation_id] = exchange
        for completion_expression in self.__completion_expression_list:
          if completion_expression.matches(exchange):
            del self.__aggregation_state[correlation_id]
            return exchange
      finally:
        self.__lock.release()
    return None






class ExchangeHandler(object):
  def __init__(self):
    self.__processor = None
    self.__filter = None
    self.__aggregate = None
    self.__endpoint = None
    self.__endpoint_configuration = None

  @property
  def processor(self):
    return self.__processor

  @processor.setter
  def processor(self, value):
    self.set_processor(value)

  def set_processor(self, value):
    self.__processor = value
    return self

  @property
  def endpoint(self):
    return self.__endpoint

  @endpoint.setter
  def endpoint(self, value):
    self.set_endpoint(value)

  def set_endpoint(self, value):
    self.__endpoint = value
    return self

  @property
  def endpoint_configuration(self):
    return self.__endpoint

  @endpoint_configuration.setter
  def endpoint_configuration(self, value):
    self.set_endpoint_configuration(value)

  def set_endpoint_configuration(self, value):
    self.__endpoint_configuration = value
    return self

  @property
  def filter(self):
    return self.__filter

  @filter.setter
  def filter(self, value):
    self.set_filter(value)

  def set_filter(self, value):
    self.__filter = value
    return self

  @property
  def aggregate(self):
    return self.__aggregate

  @aggregate.setter
  def aggregate(self, value):
    self.set_aggregate(value)

  def set_aggregate(self, value):
    self.__aggregate = value
    return self



from ..base.endpoint import Endpoint

class MockEndpoint(Endpoint):
  def __init__(self, context, original_endpoint, expected_message_count):
    super().__init__()
    self.__exchanges = context.manager.list()
    self.__expected_message_count = expected_message_count
    self.__original_endpoint = original_endpoint

  def process(self, exchange, configuration):
    self.exchanges.append(exchange)
    if self.__original_endpoint:
      self.__original_endpoint.process(exchange, configuration)

  @property
  def exchanges(self):
    return self.__exchanges

  @property
  def expected_message_count(self):
    return self.__expected_message_count